# coding=utf-8
from ..model import Account
from ..model import Group
from ..model import SurveySession
from AccessControl.SecurityManagement import newSecurityManager
from anytree import RenderTree
from euphorie.client.tests.utils import addSurvey
from euphorie.client.tests.utils import registerUserInClient
from euphorie.content.tests.utils import BASIC_SURVEY
from euphorie.testing import EuphorieFunctionalTestCase
from euphorie.testing import EuphorieIntegrationTestCase
from z3c.saconfig import Session

import datetime
import urllib


class CountryTests(EuphorieIntegrationTestCase):

    maxDiff = None

    def test_sessions_ordering(self):
        addSurvey(self.portal, BASIC_SURVEY)
        session = Session()
        account = Account(
            loginname='johny',
            sessions=[
                SurveySession(
                    zodb_path='nl/ict/software-development',
                    title=u'One',
                    modified=datetime.datetime(2012, 12, 10)
                ),
                SurveySession(
                    zodb_path='nl/ict/software-development',
                    title=u'Three',
                    modified=datetime.datetime(2012, 12, 12)
                ),
                SurveySession(
                    zodb_path='nl/ict/software-development',
                    title=u'Two',
                    modified=datetime.datetime(2012, 12, 11)
                )
            ]
        )
        session.add(account)
        newSecurityManager(None, account)
        with self._get_view('view', self.portal.client['nl']) as view:
            root = view.get_sessions_tree_root()
            self.assertListEqual(
                [x.title for x in root.descendants],
                [u'Software development', u'One', u'Three', u'Two'],
            )

            survey = root.children[0]
            # Ensure that the nodes are sorted by title
            # when getting the groups...
            self.assertListEqual(
                [x.title for x in survey.groups],
                [u'One', u'Three', u'Two'],
            )
            # ... or by reversed modification date when getting the sessions
            self.assertListEqual(
                [x.title for x in survey.sessions],
                [u'Three', u'Two', 'One'],
            )

    def test_complex_tree(self):
        # First we create a couple of surveys
        ANOTHER_BASIC_SURVEY = BASIC_SURVEY.replace(
            'Software development', 'Hardware development'
        )
        addSurvey(self.portal, BASIC_SURVEY)
        addSurvey(self.portal, ANOTHER_BASIC_SURVEY)

        # Then we create our group hierarchy
        session = Session()
        group1 = Group(group_id='1', short_name=u'Group 1')
        session.add(group1)
        group2 = Group(group_id='2', short_name=u'Group 2')
        session.add(group2)
        group2.parent = group1

        john = Account(
            loginname='john',
            sessions=[
                SurveySession(
                    zodb_path='nl/ict/software-development',
                    title=u'John SW Group 1',
                    modified=datetime.datetime(2012, 12, 10),
                    group_id=group1.group_id,
                ),
                SurveySession(
                    zodb_path='nl/ict/software-development',
                    title=u'John SW Group 2',
                    modified=datetime.datetime(2012, 12, 12),
                    group_id=group2.group_id,
                ),
                SurveySession(
                    zodb_path='nl/ict/software-development',
                    title=u'John SW No group',
                    modified=datetime.datetime(2012, 12, 11),
                ),
            ]
        )
        john.group = group1
        jane = Account(
            loginname='jane',
            sessions=[
                SurveySession(
                    zodb_path='nl/ict-1/hardware-development',
                    title=u'Jane HW No group',
                    modified=datetime.datetime(2011, 12, 10)
                ),
                SurveySession(
                    zodb_path='nl/ict-1/hardware-development',
                    title=u'Jane HW Group 2',
                    modified=datetime.datetime(2011, 12, 12),
                    group_id=group2.group_id,
                ),
                SurveySession(
                    zodb_path='nl/ict/software-development',
                    title=u'Jane SW Group 2',
                    modified=datetime.datetime(2011, 12, 11),
                    group_id=group2.group_id,
                ),
                SurveySession(
                    zodb_path='nl/ict/software-development',
                    title=u'Jane SW Group 2 (another one)',
                    modified=datetime.datetime(2011, 12, 12),
                    group_id=group2.group_id,
                ),
            ]
        )
        jane.group = group2
        session.add(john)
        session.add(jane)

        # By default John can see his sessions
        newSecurityManager(None, john)
        with self._get_view('view', self.portal.client['nl']) as view:
            root = view.get_sessions_tree_root()
            self.assertListEqual(
                RenderTree(root).by_attr("title").splitlines(),
                [
                    u'',
                    u'├── Group 1 (1)',
                    u'│   ├── Software development',
                    u'│   │   └── John SW Group 1',
                    u'│   └── Group 2 (2)',
                    u'│       └── Software development',
                    u'│           └── John SW Group 2',
                    u'└── Software development',
                    u'    └── John SW No group',
                ],
            )

        # but he can also see Jane's ones that have been filed its groups
        with self._get_view('view', self.portal.client['nl']) as view:
            view.request.set('scope', 'all')
            root = view.get_sessions_tree_root()
            self.assertListEqual(
                RenderTree(root).by_attr("title").splitlines(),
                [
                    u'',
                    u'├── Group 1 (1)',
                    u'│   ├── Software development',
                    u'│   │   └── John SW Group 1',
                    u'│   └── Group 2 (2)',
                    u'│       ├── Software development',
                    u'│       │   ├── John SW Group 2',
                    u'│       │   ├── Jane SW Group 2',
                    u'│       │   └── Jane SW Group 2 (another one)',
                    u'│       └── Hardware development',
                    u'│           └── Jane HW Group 2',
                    u'└── Software development',
                    u'    └── John SW No group',
                ],
            )
        # Jane can see her Sessions by default
        newSecurityManager(None, jane)
        with self._get_view('view', self.portal.client['nl']) as view:
            root = view.get_sessions_tree_root()
            self.assertListEqual(
                RenderTree(root).by_attr("title").splitlines(),
                [
                    u'',
                    u'├── Hardware development',
                    u'│   └── Jane HW No group',
                    u'└── Group 1 (1)',
                    u'    └── Group 2 (2)',
                    u'        ├── Hardware development',
                    u'        │   └── Jane HW Group 2',
                    u'        └── Software development',
                    u'            ├── Jane SW Group 2',
                    u'            └── Jane SW Group 2 (another one)',
                ],
            )
        # but cannot acquire John's sessions in group1
        # (although she can if the session is in group2)
        with self._get_view('view', self.portal.client['nl']) as view:
            view.request.set('scope', 'all')
            root = view.get_sessions_tree_root()
            self.assertListEqual(
                RenderTree(root).by_attr("title").splitlines(),
                [
                    u'',
                    u'├── Hardware development',
                    u'│   └── Jane HW No group',
                    u'└── Group 1 (1)',
                    u'    └── Group 2 (2)',
                    u'        ├── Hardware development',
                    u'        │   └── Jane HW Group 2',
                    u'        └── Software development',
                    u'            ├── Jane SW Group 2',
                    u'            ├── Jane SW Group 2 (another one)',
                    u'            └── John SW Group 2',
                ],
            )


class CountryFunctionalTests(EuphorieFunctionalTestCase):

    def test_surveys_filtered_by_language(self):
        survey = """<sector xmlns="http://xml.simplon.biz/euphorie/survey/1.0">
                      <title>Sector</title>
                      <survey>
                        <title>Survey</title>
                        <language>en</language>
                      </survey>
                    </sector>"""
        survey_nl = """<sector xmlns="http://xml.simplon.biz/euphorie/survey/1.0">
                    <title>Branche</title>
                    <survey>
                      <title>Vragenlijst</title>
                      <language>nl</language>
                    </survey>
                  </sector>"""  # noqa
        self.loginAsPortalOwner()
        addSurvey(self.portal, survey)
        addSurvey(self.portal, survey_nl)
        browser = self.get_browser()
        # Pass the language as URL parameter to ensure that we get the NL
        # version
        browser.open("%s?language=nl" % self.portal.client.absolute_url())
        registerUserInClient(browser, link="Registreer")
        # Note, this used to test that the URL was that of the client,
        # in the correct country (nl), with `?language=nl-NL` appended.
        # I don't see where in the code this language URL parameter would
        # come from, so I remove it in this test as well.
        self.assertEqual(browser.url, "http://nohost/plone/client/nl")
        self.assertEqual(
            browser.getControl(name="survey").options, ["branche/vragenlijst"]
        )
        browser.open(
            "%s?language=en" % self.portal.client["nl"].absolute_url()
        )
        self.assertEqual(
            browser.getControl(name="survey").options, ["sector/survey"]
        )

    def test_must_select_valid_survey(self):
        self.loginAsPortalOwner()
        addSurvey(self.portal, BASIC_SURVEY)
        browser = self.get_browser()
        browser.open(self.portal.client['nl'].absolute_url())
        registerUserInClient(browser)
        data = urllib.urlencode({
            'action': 'new',
            'survey': '',
            'title:utf8:ustring': 'Foo'
        })
        browser.handleErrors = False
        browser.open(browser.url, data)
        self.assertEqual(browser.url, 'http://nohost/plone/client/nl')



# Copied from repoze.bfg


class reify(object):
    """ Put the result of a method which uses this (non-data)
    descriptor decorator in the instance dict after the first call,
    effectively replacing the decorator with an instance variable."""

    def __init__(self, wrapped):
        self.wrapped = wrapped
        try:
            self.__doc__ = wrapped.__doc__
        except:  # pragma: no cover
            pass

    def __get__(self, inst, objtype=None):
        if inst is None:
            return self
        val = self.wrapped(inst)
        setattr(inst, self.wrapped.__name__, val)
        return val



# coding=utf-8
from ...ghost import PathGhost
from ..fti import check_fti_paste_allowed
from euphorie.content.fti import ConditionalDexterityFTI
from euphorie.content.fti import IConstructionFilter
from euphorie.content.module import Module
from euphorie.testing import EuphorieIntegrationTestCase
from plone import api
from zope.component import adapts
from zope.component import getGlobalSiteManager
from zope.component import provideAdapter
from zope.interface import implements

import mock
import unittest


class Veto(object):
    adapts(ConditionalDexterityFTI, Module)
    implements(IConstructionFilter)

    def __init__(self, fti, container):
        self.fti = fti
        self.container = container

    def allowed(self):
        return False


class ConditionalDexterityFTITests(EuphorieIntegrationTestCase):

    def _create(self, container, *args, **kwargs):
        newid = container.invokeFactory(*args, **kwargs)
        return getattr(container, newid)

    def createContent(self):
        with api.env.adopt_roles(['Member', 'Manager']):
            self.country = self.portal.sectors.nl
            self.sector = self._create(
                self.country, "euphorie.sector", "sector"
            )
            self.surveygroup = self._create(
                self.sector, "euphorie.surveygroup", "group"
            )
            self.survey = self._create(
                self.surveygroup, "euphorie.survey", "survey"
            )
            self.module = self._create(
                self.survey, "euphorie.module", "survey"
            )

    def testConditionalFtiUsedByModule(self):
        fti = getattr(self.portal.portal_types, "euphorie.module")
        self.failUnless(isinstance(fti, ConditionalDexterityFTI))

    def testVeto(self):
        self.createContent()
        provideAdapter(Veto, name="euphorie.module")
        try:
            fti = getattr(self.portal.portal_types, "euphorie.module")
            self.assertEqual(fti.isConstructionAllowed(self.module), False)
        finally:
            getGlobalSiteManager().unregisterAdapter(
                Veto, name="euphorie.module"
            )

    def testVetoRegisteredForOtherType(self):
        self.createContent()
        provideAdapter(factory=Veto, name="euphorie.solution")
        try:
            fti = getattr(self.portal.portal_types, "euphorie.module")
            self.assertEqual(fti.isConstructionAllowed(self.module), True)
        finally:
            getGlobalSiteManager().unregisterAdapter(
                Veto, name="euphorie.solution"
            )


class check_fti_paste_allowed_tests(unittest.TestCase):

    def check_fti_paste_allowed(self, *a, **kw):
        return check_fti_paste_allowed(*a, **kw)

    def test_acceptable_paste(self):
        content = mock.Mock()
        content.portal_type = 'euphorie.risk'
        fti = mock.Mock()
        fti.isConstructionAllowed.return_value = True
        with mock.patch('euphorie.content.fti.queryUtility', return_value=fti):
            self.check_fti_paste_allowed('folder', content)
            fti.isConstructionAllowed.assert_called_once_with('folder')

    def test_refuse_non_portal_content(self):
        self.assertRaises(
            ValueError, self.check_fti_paste_allowed, None, mock.Mock()
        )

    def test_do_not_use_acquisition_to_get_portal_type(self):
        parent = PathGhost('parent')
        parent.portal_type = 'euphorie.risk'
        content = PathGhost('child').__of__(parent)
        self.assertRaises(
            ValueError, self.check_fti_paste_allowed, None, content
        )

    def test_content_without_fti(self):
        content = mock.Mock()
        content.portal_type = 'euphorie.risk'
        self.assertRaises(
            ValueError, self.check_fti_paste_allowed, None, content
        )

    def test_fti_does_not_allow_construction(self):
        content = mock.Mock()
        content.portal_type = 'euphorie.risk'
        fti = mock.Mock()
        fti.isConstructionAllowed.return_value = False
        with mock.patch('euphorie.content.fti.queryUtility', return_value=fti):
            self.assertRaises(
                ValueError, self.check_fti_paste_allowed, 'folder', content
            )



# coding=utf-8
from Acquisition import aq_inner
from euphorie.client import model
from euphorie.client.navigation import FindNextQuestion
from euphorie.client.navigation import FindPreviousQuestion
from euphorie.client.navigation import getTreeData
from euphorie.client.navigation import QuestionURL
from euphorie.client.session import SessionManager
from euphorie.client.update import redirectOnSurveyUpdate
from euphorie.client.utils import HasText
from euphorie.content.interfaces import ICustomRisksModule
from euphorie.content.profilequestion import IProfileQuestion
from logging import getLogger
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from sqlalchemy import sql
from z3c.saconfig import Session

logger = getLogger(__name__)


class Mixin(object):

    def get_custom_risks(self):
        session = SessionManager.session
        query = Session.query(model.Risk).filter(
            sql.and_(
                model.Risk.is_custom_risk == 't',
                model.Risk.path.startswith(model.Module.path),
                model.Risk.session_id == session.id
            )
        ).order_by(model.Risk.id)
        return query.all()


class IdentificationView(BrowserView, Mixin):
    """The introduction page for a module.
    """
    variation_class = "variation-risk-assessment"
    phase = "identification"
    question_filter = None
    template = ViewPageTemplateFile('templates/module_identification.pt')

    def __call__(self):
        # Render the page only if the user has edit rights,
        # otherwise redirect to the start page of the session.
        if not (
            self.context.restrictedTraverse('webhelpers').can_edit_session()
        ):

            return self.request.response.redirect(
                self.context.aq_parent.aq_parent.absolute_url() + '/@@start'
            )
        if redirectOnSurveyUpdate(self.request):
            return
        context = aq_inner(self.context)
        module = self.request.survey.restrictedTraverse(
            context.zodb_path.split("/"))
        if self.request.environ["REQUEST_METHOD"] == "POST":
            self.save_and_continue(module)
        else:
            if IProfileQuestion.providedBy(module) and context.depth == 2:
                next = FindNextQuestion(context, filter=self.question_filter)
                if next is None:
                    url = "%s/actionplan" % self.request.survey.absolute_url()
                else:
                    url = QuestionURL(
                        self.request.survey, next, phase=self.phase)
                return self.request.response.redirect(url)

            # elif ICustomRisksModule.providedBy(module) \
            #         and not self.context.skip_children \
            #         and len(self.get_custom_risks()):
            #     url = "%s/customization/%d" % (
            #         self.request.survey.absolute_url(),
            #         int(self.context.path))
            #     return self.request.response.redirect(url)

            self.tree = getTreeData(
                self.request, context, filter=self.question_filter)
            self.title = module.title
            self.module = module
            number_files = 0
            for i in range(1, 5):
                number_files += getattr(
                    self.module, 'file{0}'.format(i), None) and 1 or 0
            self.has_files = number_files > 0
            self.next_is_actionplan = not FindNextQuestion(
                context, filter=self.question_filter)
            if ICustomRisksModule.providedBy(module):
                template = ViewPageTemplateFile(
                    'templates/module_identification_custom.pt'
                ).__get__(self, "")
            else:
                template = self.template
            return template()

    def save_and_continue(self, module):
        """ We received a POST request.
            Submit the form and figure out where to go next.
        """
        context = aq_inner(self.context)
        reply = self.request.form
        if module.optional:
            if "skip_children" in reply:
                context.skip_children = reply.get("skip_children")
                context.postponed = False
            else:
                context.postponed = True
            SessionManager.session.touch()

        if reply.get("next") == "previous":
            next = FindPreviousQuestion(context, filter=self.question_filter)
            if next is None:
                # We ran out of questions, step back to intro page
                url = "%s/identification" % self.request.survey.absolute_url()
                self.request.response.redirect(url)
                return
        else:
            if ICustomRisksModule.providedBy(module):
                if reply["next"] == "add_custom_risk":
                    risk_id = self.add_custom_risk()
                    url = "%s/%d" % (self.context.absolute_url(), risk_id)
                    self.request.response.redirect(url)
                    return
                else:
                    # We ran out of questions, proceed to the evaluation
                    url = "%s/actionplan" % self.request.survey.absolute_url()
                    return self.request.response.redirect(url)
            next = FindNextQuestion(context, filter=self.question_filter)
            if next is None:
                # We ran out of questions, proceed to the evaluation
                url = "%s/actionplan" % self.request.survey.absolute_url()
                return self.request.response.redirect(url)

        url = QuestionURL(self.request.survey, next, phase="identification")
        self.request.response.redirect(url)

    def add_custom_risk(self):

        session = SessionManager.session
        sql_risks = self.context.children()
        if sql_risks.count():
            counter_id = max(
                [int(risk.path[-3:]) for risk in sql_risks.all()]) + 1
        else:
            counter_id = 1

        # Add a new risk
        risk = model.Risk(
            comment="",
            priority=None,
            risk_id=None,
            risk_type='risk',
            skip_evaluation=True,
            title="",
            identification=None,
            training_notes="",
            custom_description="",
        )
        risk.is_custom_risk = True
        risk.skip_children = False
        risk.postponed = False
        risk.has_description = None
        risk.zodb_path = "/".join(
            [session.zodb_path] +
            [self.context.zodb_path] +
            # There's a constraint for unique zodb_path per session
            ['%d' % counter_id]
        )
        risk.profile_index = 0  # XXX: not sure what this is for
        self.context.addChild(risk)
        return counter_id


class ActionPlanView(BrowserView):
    """The introduction page for an :obj:`euphorie.content.module` in an action
    plan.

    """
    variation_class = "variation-risk-assessment"
    phase = "actionplan"
    question_filter = model.ACTION_PLAN_FILTER

    @property
    def use_solution_direction(self):
        module = self.request.survey.restrictedTraverse(
            self.context.zodb_path.split("/"))
        return HasText(getattr(module, "solution_direction", None))

    def __call__(self):
        # Render the page only if the user has edit rights,
        # otherwise redirect to the start page of the session.
        if not (
            self.context.restrictedTraverse('webhelpers').can_edit_session()
        ):

            return self.request.response.redirect(
                self.context.aq_parent.aq_parent.absolute_url() + '/@@start'
            )
        if redirectOnSurveyUpdate(self.request):
            return
        if self.request.environ["REQUEST_METHOD"] == "POST":
            return self._update()

        context = aq_inner(self.context)
        module = self.request.survey.restrictedTraverse(
            self.context.zodb_path.split("/"))
        self.module = module
        if (
            (IProfileQuestion.providedBy(module) and context.depth == 2) or
            (
                ICustomRisksModule.providedBy(module) and
                self.phase == 'actionplan'
            )
        ):
            next = FindNextQuestion(context, filter=self.question_filter)
            if next is None:
                if self.phase == 'identification':
                    url = "%s/actionplan" % self.request.survey.absolute_url()
                elif self.phase == 'evaluation':
                    url = "%s/actionplan" % self.request.survey.absolute_url()
                elif self.phase == 'actionplan':
                    url = "%s/report" % self.request.survey.absolute_url()
            else:
                url = QuestionURL(self.request.survey, next, phase=self.phase)
            return self.request.response.redirect(url)
        else:
            return self._update()

    def _update(self):
        survey = self.request.survey
        self.title = self.context.title
        self.tree = getTreeData(
            self.request, self.context, filter=self.question_filter,
            phase=self.phase)
        previous = FindPreviousQuestion(
            self.context, filter=self.question_filter)
        if previous is None:
            self.previous_url = "%s/%s" % (
                self.request.survey.absolute_url(), self.phase)
        else:
            self.previous_url = QuestionURL(survey, previous, phase=self.phase)

        next = FindNextQuestion(self.context, filter=self.question_filter)
        if next is None:
            self.next_url = "%s/report" % self.request.survey.absolute_url()
        else:
            self.next_url = QuestionURL(survey, next, phase=self.phase)
        return self.index()



from AccessControl import getSecurityManager
from Acquisition import aq_inner
from euphorie.client import CONDITIONS_VERSION
from euphorie.client.interfaces import IClientSkinLayer
from five import grok
from z3c.appconfig.interfaces import IAppConfig
from z3c.appconfig.utils import asBool
from zope.component import getUtility
from zope.interface import Interface

import logging


log = logging.getLogger(__name__)

grok.templatedir("templates")


def checkTermsAndConditions():
    appconfig = getUtility(IAppConfig)
    try:
        return asBool(appconfig["euphorie"]["terms-and-conditions"])
    except KeyError:
        return True
    except ValueError:
        log.error("Invalid value for terms-and-conditions flag "
                  "in site configuration.")
        return False


def approvedTermsAndConditions(account=None):
    if account is None:
        account = getSecurityManager().getUser()
    return account.tc_approved is not None and \
            account.tc_approved == CONDITIONS_VERSION


class TermsAndConditions(grok.View):
    grok.name("terms-and-conditions")
    grok.context(Interface)
    grok.require("zope2.Public")
    grok.layer(IClientSkinLayer)
    grok.template("conditions")

    def terms_changed(self):
        return getattr(self.account, 'tc_approved', None) is not None

    def update(self):
        self.came_from = self.request.form.get("came_from")
        if isinstance(self.came_from, list):
            # If came_from is both in the querystring and the form data
            self.came_from = self.came_from[0]

        self.account = getSecurityManager().getUser()
        if self.request.environ["REQUEST_METHOD"] == "POST":
            self.account.tc_approved = CONDITIONS_VERSION

            if self.came_from:
                self.request.response.redirect(self.came_from)
            else:
                self.request.response.redirect(
                        aq_inner(self.context).absolute_url())



from AccessControl.SecurityManagement import getSecurityManager
from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManagement import setSecurityManager
from Acquisition import aq_parent
from euphorie.content.interfaces import ISurveyUnpublishEvent
from euphorie.content.survey import ISurvey
from five import grok
from plonetheme.nuplone.utils import getPortal
from Products.CMFCore.utils import getToolByName

import logging


log = logging.getLogger(__name__)


@grok.subscribe(ISurvey, ISurveyUnpublishEvent)
def handleSurveyUnpublish(survey, event):
    """Event handler (subscriber) to take care of unpublishing a survey
    from the client.
    """
    surveygroup = aq_parent(survey)
    sector = aq_parent(surveygroup)
    country = aq_parent(sector)

    pas = getToolByName(survey, "acl_users")
    clientuser = pas.getUserById("client")
    sm = getSecurityManager()
    try:
        newSecurityManager(None, clientuser)
        client = getPortal(survey).client
        try:
            clientcountry = client[country.id]
            clientsector = clientcountry[sector.id]
            clientsector[surveygroup.id]
        except KeyError:
            log.info("Trying to unpublish unpublished survey %s",
                    "/".join(survey.getPhysicalPath()))
            return

        clientsector.manage_delObjects([surveygroup.id])
        if not clientsector.keys():
            clientcountry.manage_delObjects([clientsector.id])
    finally:
        setSecurityManager(sm)



# coding=utf-8
from euphorie.client.api.authentication import generate_token
from euphorie.client.api.module import Identification
from euphorie.client.api.tests.utils import _setup_session
from euphorie.client.model import Module
from euphorie.testing import EuphorieFunctionalTestCase
from euphorie.testing import EuphorieIntegrationTestCase
from sqlalchemy.orm import object_session
from zope.publisher.browser import TestRequest

import json


DUMMY_GIF = 'GIF89a\x01\x00\x01\x00\x80\x00\x00\x00\x00\x00\xff' \
            '\xff\xff!\xf9\x04\x01\x00\x00\x01\x00,\x00\x00\x00' \
            '\x00\x01\x00\x01\x00\x00\x02\x01L\x00;'


class ViewTests(EuphorieIntegrationTestCase):

    def View(self, *a, **kw):
        from euphorie.client.api.module import View
        return View(*a, **kw)

    def test_do_GET_minimal(self):
        from sqlalchemy.orm import object_session
        from zope.publisher.browser import TestRequest
        from euphorie.client.model import Module
        self.loginAsPortalOwner()
        (account, survey, survey_session) = _setup_session(self.portal)
        request = TestRequest()
        request.survey = survey
        module = object_session(survey_session).query(Module).first()
        view = self.View(module, request)
        response = view.do_GET()
        self.assertEqual(
            set(response),
            set(['id', 'type', 'title', 'description', 'optional'])
        )
        self.assertEqual(response['id'], 1)
        self.assertEqual(response['type'], 'module')
        self.assertEqual(response['title'], u'Module one')
        self.assertEqual(response['description'], u'Quick description')
        self.assertEqual(response['optional'], False)

    def test_do_GET_full(self):
        from sqlalchemy.orm import object_session
        from zope.publisher.browser import TestRequest
        from euphorie.client.model import Module
        from plone.namedfile.file import NamedBlobImage
        self.loginAsPortalOwner()
        (account, survey, survey_session) = _setup_session(self.portal)
        module = survey['1']
        module.description = u'<p>Simple description</p>'
        module.solution_direction = u'<p>Fix It Fast</p>'
        module.optional = True
        module.question = u'Is this needed?'
        module.image = NamedBlobImage(
            data=DUMMY_GIF, contentType='image/gif', filename=u'dummy.gif'
        )
        module.caption = u'Key Image'
        request = TestRequest()
        request.survey = survey
        module = object_session(survey_session).query(Module).first()
        module.skip_children = True
        view = self.View(module, request)
        response = view.do_GET()
        self.assertEqual(
            set(response),
            set([
                'id', 'type', 'title', 'optional', 'image', 'description',
                'solution-direction', 'question', 'skip-children'
            ])
        )
        self.assertEqual(response['description'], u'<p>Simple description</p>')
        self.assertEqual(response['solution-direction'], u'<p>Fix It Fast</p>')
        self.assertEqual(response['question'], u'Is this needed?')
        self.assertEqual(response['skip-children'], True)
        self.assertEqual(
            set(response['image']), set(['thumbnail', 'original', 'caption'])
        )
        self.assertEqual(response['image']['caption'], u'Key Image')
        self.assertEqual(
            response['image']['original'],
            u'http://nohost/plone/client/nl/ict/software-development/1/'
            '@@download/image/dummy.gif'
        )
        self.assertTrue(
            response['image']['thumbnail'].startswith(
                u'http://nohost/plone/client/nl/ict/software-development'
                '/1/@@images/'
            )
        )


class IdentificationTests(EuphorieIntegrationTestCase):

    def Identification(self, *a, **kw):
        return Identification(*a, **kw)

    def test_do_PUT_missing_value(self):
        self.loginAsPortalOwner()
        (account, survey, survey_session) = _setup_session(self.portal)
        module = survey['1']
        module.optional = True
        request = TestRequest()
        request.survey = survey
        request.survey_session = survey_session
        risk = object_session(survey_session).query(Module).first()
        view = self.Identification(risk, request)
        view.input = {}
        response = view.do_PUT()
        self.assertEqual(response['type'], 'error')

    def test_do_PUT_invalid_value(self):
        self.loginAsPortalOwner()
        (account, survey, survey_session) = _setup_session(self.portal)
        module = survey['1']
        module.optional = True
        request = TestRequest()
        request.survey = survey
        request.survey_session = survey_session
        risk = object_session(survey_session).query(Module).first()
        view = self.Identification(risk, request)
        view.input = {'skip-children': 'yes'}
        response = view.do_PUT()
        self.assertEqual(response['type'], 'error')

    def test_do_PUT_update_value(self):
        self.loginAsPortalOwner()
        (account, survey, survey_session) = _setup_session(self.portal)
        module = survey['1']
        module.optional = True
        request = TestRequest()
        request.survey = survey
        request.survey_session = survey_session
        module = object_session(survey_session).query(Module).first()
        view = self.Identification(module, request)
        view.input = {'skip-children': True}
        response = view.do_PUT()
        self.assertEqual(response['skip-children'], True)
        self.assertEqual(module.skip_children, True)


class BrowserTests(EuphorieFunctionalTestCase):

    def test_get(self):
        self.loginAsPortalOwner()
        (account, survey, survey_session) = _setup_session(self.portal)
        browser = self.get_browser()
        browser.addHeader('X-Euphorie-Token', generate_token(account))
        browser.open('http://nohost/plone/client/api/users/1/sessions/1/1')
        self.assertEqual(browser.headers['Content-Type'], 'application/json')
        response = json.loads(browser.contents)
        self.assertEqual(response['type'], 'module')



# coding=utf-8
"""
Country
-------

The main view after login, to list existing sessions, start new sessions,
delete & rename sessions

URL: https://client-oiranew.syslab.com/eu
"""

from .. import MessageFactory as _
from AccessControl import getSecurityManager
from euphorie.client.interfaces import IClientSkinLayer
from euphorie.client.model import SurveySession
from five import grok
from plone.app.dexterity.behaviors.metadata import IBasic
from plone.directives import dexterity
from plone.directives import form
from Products.statusmessages.interfaces import IStatusMessage
from sqlalchemy.orm import object_session
from z3c.form import button
from z3c.saconfig import Session
from zope import schema
from zope.interface import directlyProvides
from zope.interface import implementer

import logging


grok.templatedir("templates")

log = logging.getLogger(__name__)


class IClientCountry(form.Schema, IBasic):
    """Country grouping in the online client.
    """


@implementer(IClientCountry)
class ClientCountry(dexterity.Container):

    country_type = None


class DeleteSession(grok.View):
    """View name: @@delete-session
    """
    grok.context(IClientCountry)
    grok.require("euphorie.client.ViewSurvey")
    grok.layer(IClientSkinLayer)
    grok.name("delete-session")

    def render(self):
        session = Session()
        ss = session.query(SurveySession).get(self.request.form["id"])
        if ss is not None:
            flash = IStatusMessage(self.request).addStatusMessage
            flash(
                _(
                    u"Session `${name}` has been deleted.",
                    mapping={
                        "name": getattr(ss, 'title')
                    }
                ), "success"
            )
            session.delete(ss)
        self.request.response.redirect(self.context.absolute_url())


class RenameSessionSchema(form.Schema):
    title = schema.TextLine(required=False)


class RenameSession(form.SchemaForm):
    """View name: @@rename-session
    """
    grok.context(IClientCountry)
    grok.require("euphorie.client.ViewSurvey")
    grok.layer(IClientSkinLayer)
    grok.name("rename-session")
    grok.template("rename-session")
    form.wrap(False)

    schema = RenameSessionSchema

    def getContent(self):
        try:
            session_id = int(self.request.get("id"))
        except (ValueError, TypeError):
            raise KeyError("Invalid session id")
        user = getSecurityManager().getUser()
        session = (
            object_session(user).query(SurveySession)
            .filter(SurveySession.account == user)
            .filter(SurveySession.id == session_id).first()
        )
        if session is None:
            raise KeyError("Unknown session id")
        self.original_title = session.title
        directlyProvides(session, RenameSessionSchema)
        return session

    @button.buttonAndHandler(_(u"Save"))
    def handleSave(self, action):
        (data, errors) = self.extractData()
        if errors:
            return
        if data["title"]:
            flash = IStatusMessage(self.request).addStatusMessage
            self.getContent().title = data['title']
            flash(
                _(
                    u"Session title has been changed to ${name}",
                    mapping={
                        "name": data["title"]
                    }
                ), "success"
            )
        came_from = self.request.form.get("came_from")
        if isinstance(came_from, list):
            # If came_from is both in the querystring and the form data
            came_from = came_from[0]
        if came_from:
            self.request.response.redirect(came_from)
        else:
            self.response.redirect(self.context.absolute_url())

    @button.buttonAndHandler(_(u"Cancel"))
    def handleCancel(self, action):
        self.response.redirect(self.context.absolute_url())



# coding=utf-8
from ...tests.utils import createSector
from ..authentication import generate_token
from euphorie.testing import EuphorieFunctionalTestCase

import json


class BrowserAPITests(EuphorieFunctionalTestCase):

    def test_get_version(self):
        browser = self.get_browser()
        browser.open('http://nohost/plone/api')
        self.assertEqual(browser.headers['Content-Type'], 'application/json')
        response = json.loads(browser.contents)
        self.assertEqual(
            set(response), set(['api-version', 'euphorie-version', 'account'])
        )
        self.assertEqual(response['api-version'], [1, 0])
        self.assertEqual(response['account'], None)

    def test_authenticated_user(self):
        sector = createSector(self.portal, login='sector', password=u'sector')
        browser = self.get_browser()
        browser.addHeader('X-Euphorie-Token', generate_token(sector))
        browser.open('http://nohost/plone/api')
        response = json.loads(browser.contents)
        self.assertEqual(response['account'], 'sector')
